ProvinatusCompanion = ZO_Object:Subclass()

-- Actual drawing of icon happens in Teams.lua. This is just to present a separate menu.
function ProvinatusCompanion:New(...)
  return ZO_Object.New(self)
end

function ProvinatusCompanion:Update()
  local Elements = {}
  if Provinatus:GetSettings("Companion").Enabled and HasActiveCompanion() then
    local X, Y, _, IsInCurrentMap = GetMapPlayerPosition("companion")
    if IsInCurrentMap then
      local Element = {
        X = X,
        Y = Y,
        Alpha = Provinatus:GetSettings("Companion").Alpha,
        Size = Provinatus:GetSettings("Companion").Size,
        Texture = ZO_MapPin.PIN_DATA[MAP_PIN_TYPE_ACTIVE_COMPANION].texture
      }

      table.insert(Elements, Element)
    end
  end

  Provinatus:DrawElements(self, Elements)
end

function ProvinatusCompanion:GetMenu()
  local function getSize()
    return Provinatus.SavedVars.Companion.Size
  end

  local function setSize(value)
    Provinatus.SavedVars.Companion.Size = value
  end

  local function getAlpha()
    return Provinatus.SavedVars.Companion.Alpha * 100
  end

  local function setAlpha(value)
    Provinatus.SavedVars.Companion.Alpha = value / 100
  end

  local Controls = {
    {
      type = "checkbox",
      name = PROVINATUS_ENABLE,
      getFunc = function()
        return Provinatus.SavedVars.Companion.Enabled
      end,
      setFunc = function(value)
        Provinatus.SavedVars.Companion.Enabled = value
      end,
      width = "full",
      tooltip = PROVINATUS_COMPANION_ENABLE_TT,
      default = ProvinatusConfig.Companion.Enabled
    },
    {
      type = "slider",
      name = PROVINATUS_ICON_SIZE,
      getFunc = getSize,
      setFunc = setSize,
      min = 20,
      max = 150,
      step = 1,
      clampInput = true,
      decimals = 0,
      autoSelect = true,
      inputLocation = "below",
      tooltip = PROVINATUS_ICON_SIZE_TT,
      width = "half",
      default = ProvinatusConfig.Companion.Size
    },
    {
      type = "slider",
      name = PROVINATUS_TRANSPARENCY,
      getFunc = getAlpha,
      setFunc = setAlpha,
      min = 0,
      max = 100,
      step = 1,
      clampInput = true,
      decimals = 0,
      autoSelect = true,
      inputLocation = "below",
      tooltip = PROVINATUS_TRANSPARENCY_TT,
      width = "half",
      default = ProvinatusConfig.Companion.Alpha * 100
    }
  }

  return {
    type = "submenu",
    name = SI_COMPANION_MENU_ROOT_TITLE,
    controls = Controls,
    icon = ZO_MapPin.PIN_DATA[MAP_PIN_TYPE_ACTIVE_COMPANION].texture
  }
end
